"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AutoTransformer = void 0;
const graphql_1 = require("graphql");
const graphql_transformer_core_1 = require("graphql-transformer-core");
const graphql_transformer_common_1 = require("graphql-transformer-common");
const graphql_mapping_template_1 = require("graphql-mapping-template");
class AutoTransformer extends graphql_transformer_core_1.Transformer {
    constructor() {
        super('AutoTransformer', graphql_transformer_core_1.gql `
        directive @auto(creatable: Boolean = false, updatable: Boolean = false) on FIELD_DEFINITION
      `);
        this.field = (parent, definition, directive, ctx) => {
            var _a, _b;
            if (parent.kind === graphql_1.Kind.INTERFACE_TYPE_DEFINITION) {
                throw new graphql_transformer_core_1.InvalidDirectiveError(`The @auto directive cannot be placed on an interface's field. See ${parent.name.value}${definition.name.value}`);
            }
            const modelDirective = (_a = parent.directives) === null || _a === void 0 ? void 0 : _a.find((dir) => dir.name.value === 'model');
            if (!modelDirective) {
                throw new graphql_transformer_core_1.InvalidDirectiveError('Types annotated with @auto must also be annotated with @model.');
            }
            if (!graphql_transformer_common_1.isNonNullType(definition.type)) {
                throw new graphql_transformer_core_1.TransformerContractError(`@auto directive can only be used on non-nullable type fields`);
            }
            const isArg = (s) => (arg) => arg.name.value === s;
            const getArg = (directive, arg, dflt) => {
                var _a;
                const argument = (_a = directive.arguments) === null || _a === void 0 ? void 0 : _a.find(isArg(arg));
                return argument ? graphql_1.valueFromASTUntyped(argument.value) : dflt;
            };
            const typeName = parent.name.value;
            const creatable = getArg(directive, 'creatable', false);
            const updatable = getArg(directive, 'updatable', false);
            this.updateCreateInput(ctx, typeName, definition, creatable);
            this.updateUpdateInput(ctx, typeName, definition, updatable);
            // @key directive generates VTL code before @model does.
            // There're three cases @key trie to use automatic variables before they're defined.
            // 1. An automatic generated variable is a part of composite primary key:
            //   @key(fields: ["hash", "createdAt"]
            // 2. An automatic generated variable is a part of range key:
            //   @key(name: "byThing", fields: ["hash", "sender", "createdAt"]
            // 3. An automatic generated variable satisfy 1 and 2.
            // To handle this problem, We generate automatic variables by ourselves.
            const keyDirectives = (_b = parent.directives) === null || _b === void 0 ? void 0 : _b.filter((dir) => dir.name.value === 'key');
            if (keyDirectives) {
                let useCreatedAtField = false;
                let useUpdatedAtField = false;
                let useTypeName = false;
                let createdAtField = null;
                let updatedAtField = null;
                const timestamps = getArg(modelDirective, 'timestamps');
                switch (typeof timestamps) {
                    case 'object':
                        if (timestamps !== null) {
                            createdAtField = timestamps['createdAt'];
                            updatedAtField = timestamps['updatedAt'];
                            if (createdAtField === undefined) {
                                createdAtField = 'createdAt';
                            }
                            if (updatedAtField === undefined) {
                                updatedAtField = 'updatedAt';
                            }
                        }
                        break;
                    case 'undefined':
                        createdAtField = 'createdAt';
                        updatedAtField = 'updatedAt';
                        break;
                    default:
                        throw new Error('unreachable');
                }
                for (const kd of keyDirectives) {
                    const fields = getArg(kd, 'fields');
                    if (fields) {
                        if (fields.includes(createdAtField)) {
                            useCreatedAtField = true;
                        }
                        if (fields.includes(updatedAtField)) {
                            useUpdatedAtField = true;
                        }
                        if (fields.includes('__typename')) {
                            useTypeName = true;
                        }
                    }
                }
                // Update create and update mutations
                const createResolverResourceId = graphql_transformer_common_1.ResolverResourceIDs.DynamoDBCreateResolverResourceID(typeName);
                this.updateResolver(ctx, createResolverResourceId, graphql_mapping_template_1.printBlock(`Prepare DynamoDB PutItem Request for @auto`)(graphql_mapping_template_1.compoundExpression([
                    ...(useCreatedAtField && createdAtField
                        ? [
                            graphql_mapping_template_1.qref(`$context.args.input.put("${createdAtField}", $util.defaultIfNull($ctx.args.input.${createdAtField}, util.time.nowISO8601()))`),
                        ]
                        : []),
                    ...(useUpdatedAtField && updatedAtField
                        ? [
                            graphql_mapping_template_1.qref(`$context.args.input.put("${updatedAtField}", $util.defaultIfNull($ctx.args.input.${updatedAtField}, util.time.nowISO8601()))`),
                        ]
                        : []),
                    ...(useTypeName ? [graphql_mapping_template_1.qref(`$context.args.input.put("__typename", "${typeName}")`)] : []),
                ])));
                const updateResolverResourceId = graphql_transformer_common_1.ResolverResourceIDs.DynamoDBUpdateResolverResourceID(typeName);
                this.updateResolver(ctx, updateResolverResourceId, graphql_mapping_template_1.printBlock(`Prepare DynamoDB UpdateItem Request for @auto`)(graphql_mapping_template_1.compoundExpression([
                    ...(useUpdatedAtField && updatedAtField
                        ? [
                            graphql_mapping_template_1.qref(`$context.args.input.put("${updatedAtField}", $util.defaultIfNull($ctx.args.input.${updatedAtField}, util.time.nowISO8601()))`),
                        ]
                        : []),
                    ...(useTypeName ? [graphql_mapping_template_1.qref(`$context.args.input.put("__typename", "${typeName}")`)] : []),
                ])));
            }
        };
        this.updateResolver = (ctx, resolverResourceId, code) => {
            const resolver = ctx.getResource(resolverResourceId);
            if (resolver) {
                const templateParts = [code, resolver.Properties.RequestMappingTemplate];
                resolver.Properties.RequestMappingTemplate = templateParts.join('\n\n');
                ctx.setResource(resolverResourceId, resolver);
            }
        };
    }
    updateCreateInput(ctx, typeName, autoField, nullable) {
        this.updateInput(ctx, graphql_transformer_common_1.ModelResourceIDs.ModelCreateInputObjectName(typeName), typeName, autoField, nullable);
    }
    updateUpdateInput(ctx, typeName, autoField, nullable) {
        this.updateInput(ctx, graphql_transformer_common_1.ModelResourceIDs.ModelUpdateInputObjectName(typeName), typeName, autoField, nullable);
    }
    updateInput(ctx, inputName, typeName, autoField, nullable) {
        const input = ctx.getType(inputName);
        if (input && input.kind === graphql_1.Kind.INPUT_OBJECT_TYPE_DEFINITION) {
            if (input.fields) {
                if (nullable) {
                    // make autoField nullable
                    ctx.putType({
                        ...input,
                        fields: input.fields.map((f) => {
                            if (f.name.value === autoField.name.value) {
                                return graphql_transformer_common_1.makeInputValueDefinition(autoField.name.value, graphql_transformer_common_1.unwrapNonNull(autoField.type));
                            }
                            return f;
                        }),
                    });
                }
                else {
                    // or strip autoField
                    const updatedFields = input.fields.filter((f) => f.name.value !== autoField.name.value);
                    if (updatedFields.length === 0) {
                        throw new graphql_transformer_core_1.InvalidDirectiveError(`After stripping away version field "${autoField.name.value}", \
                        the create input for type "${typeName}" cannot be created \
                        with 0 fields. Add another field to type "${typeName}" to continue.`);
                    }
                    ctx.putType({
                        ...input,
                        fields: updatedFields,
                    });
                }
            }
        }
    }
}
exports.AutoTransformer = AutoTransformer;
//# sourceMappingURL=AutoTransformer.js.map